@@ -5,7 +5,7 @@ use strict 'vars';
 
 package Text::Bidi;
 # ABSTRACT: Unicode bidi algorithm using libfribidi
-$Text::Bidi::VERSION = '2.09';
+$Text::Bidi::VERSION = '2.10';
 use Exporter;
 use base qw(Exporter);
 use Carp;
@@ -129,6 +129,25 @@ sub get_par_embedding_levels {
 }
 
 
+sub join_arabic {
+    my $self = S(@_);
+    my ($t, $l, $j) = @_;
+    $self->tie_byte(Text::Bidi::private::join_arabic($$t, $$l, $$j));
+}
+
+
+sub shaped {
+    my $self = S(@_);
+    my ($flags, $el, $prop, $u) = @_;
+    return ($prop, $u) unless defined $flags;
+    $flags ||= $Text::Bidi::privatec::FRIBIDI_FLAGS_ARABIC;
+    my ($p, $v) =Text::Bidi::private::shape_arabic($flags, $$el, $$prop, 
+        $$u);
+    ($self->tie_byte($p), $self->tie_long($v))
+}
+
+
+
 sub mirrored {
     my $self = S(@_);
     my ($el, $u) = @_;
@@ -238,7 +257,7 @@ Text::Bidi - Unicode bidi algorithm using libfribidi
 
 =head1 VERSION
 
-version 2.09
+version 2.10
 
 =head1 SYNOPSIS
 
@@ -458,7 +477,7 @@ utf8_to_internal(). Wraps fribidi_get_bidi_types(3).
     $types = $tb->get_joining_types($internal);
 
 Returns a L<Text::Bidi::Array::Byte> with the list of joining types of the 
-text given by $internal, a representation of the paragraph text, as returned 
+text given by B<$internal>, a representation of the paragraph text, as returned 
 by L</utf8_to_internal>. Wraps fribidi_get_joining_types(3).
 
 =head2 get_joining_type_name
@@ -481,6 +500,29 @@ not given, it defaults to C<FRIBIDI_PAR_ON> (neutral).
 The output is the resolved paragraph direction I<$odir>, and the 
 L<Text::Bidi::Array::Byte> array I<$lvl> of embedding levels.
 
+=head2 join_arabic
+
+    $props = $tb->join_arabic($bidi_types, $lvl, $join_types);
+
+Returns a L<Text::Bidi::Array::Byte> with B<$props>, as returned by 
+fribidi_join_arabic(3). The inputs are B<$bidi_types>, as returned by 
+L</get_bidi_types>, B<$lvl>, as returned by 
+L</get_par_embedding_levels>, and B<$join_types> as returned by
+L</get_joining_types>.  Wraps fribidi_join_arabic(3).
+
+=head2 shaped
+
+    ($newp, $shaped) = $tb->shaped($flags, $lvl, $prop, $internal);
+
+Returns the internal representation of the paragraph, with shaping applied.  
+The internal representation of the original paragraph (as returned by 
+L</utf8_to_internal>) should be passed in B<$internal>, while the embedding 
+levels (as returned by L</get_par_embedding_levels>) should be in B<$lvl>. 
+See the documentation of F<fribidi-arabic.h> for B<$flags>, but as a special
+case, a value of C<undef> here skips shaping (returning B<($prop, $internal)>),
+while any other false value becomes the default. B<$prop> is as 
+returned by L</join_arabic>.  This method wraps fribidi_shape_arabic(3).
+
 =head2 mirrored
 
     $mirrored = $tb->mirrored($lvl, $internal);
