@@ -4,7 +4,7 @@ use integer;
 use strict 'vars';
 package Text::Bidi::Constants;
 # ABSTRACT: Constants for Text::Bidi
-$Text::Bidi::Constants::VERSION = '2.09';
+$Text::Bidi::Constants::VERSION = '2.10';
 
 use Text::Bidi::private;
 SYM: for my $sym ( keys %Text::Bidi::private:: ) {
@@ -32,6 +32,8 @@ SYM: for my $sym ( keys %Text::Bidi::private:: ) {
     }
 }
 
+*{"Text::Bidi::Flag::ARABIC"} = *{"Text::Bidi::private::FRIBIDI_FLAGS_ARABIC"};
+
 
 1;
 
@@ -45,7 +47,7 @@ Text::Bidi::Constants - Constants for Text::Bidi
 
 =head1 VERSION
 
-version 2.09
+version 2.10
 
 =head1 DESCRIPTION
 
@@ -79,7 +81,8 @@ possible constants.
 
 Constants of the form B<FRIBIDI_FLAG_FOO> are converted to 
 C<$Text::Bidi::Flag::FOO>. See fribidi_reorder_line(3) and fribidi_shape(3) 
-for possible constants.
+for possible constants. As a special case, B<FRIBIDI_FLAGS_ARABIC> becomes 
+C<$Text::Bidi::Flag::ARABIC>
 
 =item *
 
