@@ -1,5 +1,5 @@
 # Created: Tue 27 Aug 2013 04:10:03 PM IDT
-# Last Changed: Thu 17 Oct 2013 10:29:42 AM IDT
+# Last Changed: Tue 13 Oct 2015 12:05:50 PM IDT
 
 use 5.10.0;
 use warnings;
@@ -9,7 +9,7 @@ use strict;
 
 package Text::Bidi::Paragraph;
 # ABSTRACT: Run the bidi algorithm on one paragraph
-$Text::Bidi::Paragraph::VERSION = '2.09';
+$Text::Bidi::Paragraph::VERSION = '2.10';
 
 use Text::Bidi;
 
@@ -21,6 +21,8 @@ sub new {
     my @bd = ($self->{'bd'});
     $self->{'bd'} = Text::Bidi::S(@bd);
     $self->{'par'} = $par;
+    $self->{'shape'} = 0 
+      if $par =~ /[\p{bc=AL}\p{bc=AN}]/ and not exists $self->{'shape'};
     bless $self => $class;
     $self->_init
 }
@@ -57,6 +59,8 @@ sub _init {
     ($self->{'dir'}, $self->{'_levels'}) =
         $bd->get_par_embedding_levels($self->types, $self->dir);
     $self->{'_map'} = [0..$#{$self->_unicode}];
+    $self->{'_unicode'} = $self->shaped($self->{'shape'}) 
+        if defined $self->{'shape'};
     $self->{'_mirrored'} = $bd->mirrored($self->levels, $self->_unicode);
     $self->{'_mirpar'} = $bd->internal_to_utf8($self->_mirrored);
     $self->{'_par'} = [split '', $self->_mirpar ];
@@ -64,6 +68,21 @@ sub _init {
 }
 
 
+sub ar_props {
+    my ($self) = (@_);
+    $self->{'_ar_props'} //= $self->bd->join_arabic($self->types, $self->levels, $self->bd->get_joining_types($self->_unicode))
+}
+
+
+sub shaped {
+    my ($self, $flags) = (@_);
+    ($self->{'_ar_props'}, $self->{'_shaped'}) = $self->bd->shaped(
+        $flags, $self->levels, $self->ar_props, $self->_unicode)
+        unless defined $self->{'_shaped'};
+    $self->{'_shaped'}
+}
+
+
 sub visual {
     my ($self, $off, $len, $flags) = @_;
     $off //= 0;
@@ -95,7 +114,7 @@ Text::Bidi::Paragraph - Run the bidi algorithm on one paragraph
 
 =head1 VERSION
 
-version 2.09
+version 2.10
 
 =head1 SYNOPSIS
 
@@ -133,10 +152,14 @@ reordering.
 
 Create a new object corresponding to a text B<$logical> in logical order. The 
 other arguments are key-value pairs. The only ones that have a meaning at the 
-moment are I<bd>, which supplies the L<Text::Bidi> object to use, and 
-I<dir>, which prescribes the direction of the paragraph. The value of I<dir> 
+moment are I<bd>, which supplies the L<Text::Bidi> object to use, 
+I<dir>, which prescribes the direction of the paragraph, and I<shape>,
+which determines shaping flags. The value of I<dir> 
 is a constant in C<Text::Bidi::Par::> (e.g., C<$Text::Bidi::Par::RTL>; see 
-L<Text::Bidi::Constants>).
+L<Text::Bidi::Constants>). The value of I<shape> is a constant from
+fribidi_shape_arabic(3). If it is C<undef>, no shaping is done. If it is 
+missing, default shaping will be performed if the paragraph contains Arabic 
+text.
 
 Note that the mere creation of B<$par> runs the bidi algorithm on the given 
 text B<$logical> up to the point of reordering (which is dealt with in 
@@ -205,6 +228,18 @@ Returns the list of bidi types as strings
 
 Returns true if the direction of the paragraph is C<RTL> (right to left).
 
+=head2 ar_props
+
+    $props = $self->ar_props
+
+Return the shaping properties (TODO)
+
+=head2 shaped
+
+    $shaped = $self->shaped(flags)
+
+Return the shaped paragraph, and fix ar_props (TODO)
+
 =head2 visual
 
     my $visual = $par->visual($offset, $length, $flags);
